@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 
 import nbformat
 from nbclient.client import (
@@ -14,6 +14,10 @@ from .nb_result import NotebookError, NotebookResult
 NB_VERSION = 4
 
 
+class CellImportError(Exception):
+    pass
+
+
 class NotebookRun:
     filename: Path
     verbose: bool
@@ -24,11 +28,13 @@ class NotebookRun:
         default_timeout: int,
         verbose: bool = False,
         kernel: Optional[str] = None,
+        find_import_errors: bool = False,
     ) -> None:
         self.filename = filename
         self.verbose = verbose
         self.default_timeout = default_timeout
         self.kernel = kernel
+        self.find_import_errors = find_import_errors
 
     def execute(
         self,
@@ -56,7 +62,8 @@ class NotebookRun:
             c = NotebookClient(
                 nb,
                 timeout=timeout,
-                allow_errors=allow_errors,
+                allow_errors=allow_errors or self.find_import_errors,
+                interrupt_on_timeout=self.find_import_errors,
                 record_timing=True,
                 **extra_kwargs,
             )
@@ -64,23 +71,48 @@ class NotebookRun:
             async def apply_mocks(
                 cell: NotebookNode, cell_index: int, execute_reply: Dict[str, Any]
             ):
-                if c.kc is None:
-                    raise Exception("there is no kernelclient")
-                mocks: Dict[str, Any] = (
-                    cell.get("metadata", {}).get("nbmake", {}).get("mock", {})
-                )
-                for v in mocks:
-                    if isinstance(mocks[v], str):
-                        out = await c.kc.execute_interactive(f"{v} = '{mocks[v]}'")
-                    else:
-                        out = await c.kc.execute_interactive(f"{v} = {mocks[v]}")
+                # https://github.com/treebeardtech/nbmake/issues/77
+                if any(o["output_type"] == "error" for o in cell["outputs"]):
+                    execute_reply["content"]["status"] = "error"
 
-                    if out["content"]["status"] != "ok":
-                        raise Exception(f"Failed to apply mock {v}\n\n{str(out)}")
+                    if "ename" in execute_reply["content"]:
+                        if execute_reply["content"]["ename"] == "ModuleNotFoundError":
+                            if self.find_import_errors:
+                                raise CellImportError()
+                else:
+                    if c.kc is None:
+                        raise Exception("there is no kernelclient")
+                    mocks: Dict[str, Any] = (
+                        cell.get("metadata", {}).get("nbmake", {}).get("mock", {})
+                    )
+                    for v in mocks:
+                        if isinstance(mocks[v], str):
+                            out = await c.kc.execute_interactive(f"{v} = '{mocks[v]}'")
+                        else:
+                            out = await c.kc.execute_interactive(f"{v} = {mocks[v]}")
+
+                        if out["content"]["status"] != "ok":
+                            raise Exception(f"Failed to apply mock {v}\n\n{str(out)}")
+
+                    post_cell_execute: List[str] = (
+                        cell.get("metadata", {})
+                        .get("nbmake", {})
+                        .get("post_cell_execute", [])
+                    )
+                    if post_cell_execute:
+                        pce = "\n".join(post_cell_execute)
+                        out = await c.kc.execute_interactive(pce)
+
+                        if out["content"]["status"] != "ok":
+                            raise Exception(
+                                f"Post cell execution failed:\n{pce}\n\n{str(out)}"
+                            )
 
             c.on_cell_executed = apply_mocks
 
             c.execute(cwd=self.filename.parent)
+        except CellImportError:
+            error = self._get_error(nb)
         except CellExecutionError:
             error = self._get_error(nb)
         except CellTimeoutError as err:
